import logging

import structlog


def getLogger(logger=None):
    if not logger:
        logger = logging.getLogger()

    if not structlog.is_configured():
        configure_structlog()

    logger = structlog.wrap_logger(logger=logger)
    return logger


def configure_structlog():
    """
    default configuration for structlog if not specified during initialization
    :return:
    """
    structlog.configure(
        processors=[
            structlog.processors.TimeStamper(fmt="ISO"),
            structlog.processors.JSONRenderer(),
        ],
        context_class=structlog.threadlocal.wrap_dict(dict),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )



import logging
import uuid

import structlog
import wrapt

from casualty import constants

logger = logging.getLogger()


def patch():
    wrapt.wrap_function_wrapper("kombu", "messaging.Producer.publish", _inject_header)
    wrapt.wrap_function_wrapper(
        "kombu", "messaging.Consumer.__init__", _initialize_structlog_configuration
    )
    wrapt.wrap_function_wrapper(
        "kombu", "messaging.Consumer.receive", _bind_request_id_on_message_receive
    )


def _inject_header(wrapped, instance, args, kwargs):
    if "type" in args[0] and args[0]["type"] == "worker-heartbeat":
        return wrapped(*args, **kwargs)

    if "headers" in kwargs:
        headers = kwargs["headers"]
    else:
        headers = {}

    try:
        headers[constants.REQUEST_HEADER] = str(
            structlog.get_config()["context_class"]._tl.dict_["request_id"]
        )
    except Exception as e:
        headers[constants.REQUEST_HEADER] = str(uuid.uuid4())
    kwargs["headers"] = headers
    return wrapped(*args, **kwargs)


def _initialize_structlog_configuration(wrapped, instance, args, kwargs):
    structlog.configure(
        processors=[
            structlog.processors.TimeStamper(fmt="ISO"),
            structlog.processors.JSONRenderer(),
        ],
        context_class=structlog.threadlocal.wrap_dict(dict),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )

    return wrapped(*args, **kwargs)


def _bind_request_id_on_message_receive(wrapped, instance, args, kwargs):
    logger = structlog.getLogger()
    try:
        request_id = args[1].headers[constants.REQUEST_HEADER]
    except Exception as e:
        request_id = str(uuid.uuid4())

    logger = logger.bind(request_id=request_id)
    return wrapped(*args, **kwargs)



import structlog
from mock import Mock

from casualty.constants import REQUEST_HEADER
from casualty.ext.kombu.patch import _inject_header, _bind_request_id_on_message_receive


def mocked_function():
    return True


def test_inject_header(mocker):
    wrapped = Mock()
    instance = Mock()
    wrapped.__call__ = mocked_function.__call__
    args0 = {}
    kwargs = {}
    args = [args0]
    mocker.patch("uuid.uuid4", return_value="12345")
    _inject_header(wrapped, instance, args, kwargs)

    assert kwargs['headers'][REQUEST_HEADER] == '12345'


def test_bind_request_id_on_message_receive(mocker):
    wrapped = Mock()
    instance = Mock()
    wrapped.__call__ = mocked_function.__call__
    args0 = {}
    kwargs = {}
    args = [args0]
    mocker.patch("uuid.uuid4", return_value="12345")

    structlog.configure(
        processors=[
            structlog.processors.TimeStamper(fmt="ISO"),
            structlog.processors.JSONRenderer(),
        ],
        context_class=structlog.threadlocal.wrap_dict(dict),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )
    _bind_request_id_on_message_receive(wrapped, instance, args, kwargs)

    request_id = structlog.get_config()["context_class"]._tl.dict_["request_id"]
    assert request_id == '12345'



import logging
import uuid

import structlog
import wrapt

from casualty import constants

logger = logging.getLogger()


def patch():
    wrapt.wrap_function_wrapper("requests", "Session.prepare_request", _inject_header)


def _inject_header(wrapped, instance, args, kwargs):
    """

    Add request_id header to all outgoing http request

    :param wrapped:
    :param instance:
    :param args:
    :param kwargs:
    :return:
    """
    request = args[0]
    headers = getattr(request, "headers", {})
    inject_request_id_header(headers)
    setattr(request, "headers", headers)
    return wrapped(*args, **kwargs)


def inject_request_id_header(headers):
    try:
        headers[constants.REQUEST_HEADER] = str(
            structlog.get_config()["context_class"]._tl.dict_["request_id"]
        )
    except Exception as e:
        headers[constants.REQUEST_HEADER] = str(uuid.uuid4())



from mock import Mock

from casualty.constants import REQUEST_HEADER
from casualty.ext.requests.patch import _inject_header


def mocked_function():
    return True


def test_inject_header(mocker):
    wrapped = Mock()
    instance = Mock()
    wrapped.__call__ = mocked_function.__call__
    request = Mock()
    request.headers = {}
    args = [request]
    mocker.patch("uuid.uuid4", return_value="12345")
    _inject_header(wrapped, instance, args, {})
    assert request.headers[REQUEST_HEADER] == '12345'

    print(123)



import uuid

import structlog

from casualty.casualty_logger import configure_structlog
from casualty.constants import HTTP_REQUEST_HEADER


class FlaskCorelationMiddleWare(object):
    """
    If request_id header is present bind it to logger
    else create an ew request_id and bind it to logger
    It uses structlog to maintain request_id
    """

    def __init__(self, app):
        self.app = app
        if not structlog.is_configured():
            configure_structlog()

    def __call__(self, environ, start_response):

        """
        :param environ:
        :param start_response:
        :return:
        """

        logger = structlog.getLogger()
        if HTTP_REQUEST_HEADER in environ:
            current_request_id = environ[HTTP_REQUEST_HEADER]

        else:
            current_request_id = str(uuid.uuid4())
        logger = logger.bind(request_id=current_request_id)
        return self.app(environ, start_response)



import logging
import uuid
import structlog


class RequestIDFilter(logging.Filter):

    def filter(self, record):
        try:
            record.request_id = str(
                structlog.get_config()["context_class"]._tl.dict_["request_id"]
            )
        except :
            record.request_id=str(uuid.uuid4())

        return True



import importlib
import logging

log = logging.getLogger(__name__)

SUPPORTED_MODULES = ("requests", "kombu")

NO_DOUBLE_PATCH = (
    "botocore",
    "pynamodb",
    "requests",
    "sqlite3",
    "mysql",
    "pymongo",
    "psycopg2",
)

_PATCHED_MODULES = set()


def patch_all(double_patch=False):
    if double_patch:
        patch(SUPPORTED_MODULES, raise_errors=False)
    else:
        patch(NO_DOUBLE_PATCH, raise_errors=False)


def patch(modules_to_patch, raise_errors=True):
    modules = set()
    for module_to_patch in modules_to_patch:
        modules.add(module_to_patch)
    unsupported_modules = modules - set(SUPPORTED_MODULES)
    if unsupported_modules:
        raise Exception(
            "modules %s are currently not supported for patching"
            % ", ".join(unsupported_modules)
        )

    for m in modules:
        _patch_module(m, raise_errors)


def _patch_module(module_to_patch, raise_errors=True):
    try:
        _patch(module_to_patch)
    except Exception:
        if raise_errors:
            raise
        log.debug("failed to patch module %s", module_to_patch)


def _patch(module_to_patch):
    path = "casualty.ext.%s" % module_to_patch

    if module_to_patch in _PATCHED_MODULES:
        log.debug("%s already patched", module_to_patch)

    imported_module = importlib.import_module(path)
    imported_module.patch()

    _PATCHED_MODULES.add(module_to_patch)
    log.info("successfully patched module %s", module_to_patch)



from setuptools import find_packages, setup


setup(
    name="casualty",
    version="0.1.9",
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "structlog==18.2.0",
        "wrapt==1.10.11",
        "pre-commit-hooks==1.4.0",
        "mock==2.0.0",
        "pytest==3.8.2",
        "pytest-mock==1.10.0",
        "pytest-cov"
    ],
    url="",
    python_requires=">=2.6, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    license="MIT",
    author="Sohit Kumar",
    author_email="sumitk002@gmail.com",
    test_suite="tests",
    description="A python library to generate co-relation id and bind it to headers in outgoing request",
)



import uuid

import structlog

from casualty.casualty_logger import configure_structlog
from casualty.constants import HTTP_REQUEST_HEADER



def setup_request_id(request):
    logger = structlog.getLogger()
    if request.META.get(HTTP_REQUEST_HEADER):
        current_request_id = request.META.get(HTTP_REQUEST_HEADER)

    else:
        current_request_id = str(uuid.uuid4())
    logger = logger.bind(request_id=current_request_id)




class DjangoCorelationMiddleware(object):
    """
    If request_id header is present bind it to logger
    else create an ew request_id and bind it to logger
    It uses
    """

    def __init__(self, get_response):
        self.get_response = get_response
        if not structlog.is_configured():
            configure_structlog()

    def __call__(self, request):
        setup_request_id(request)

        response = self.get_response(request)

        return response


class DjangoCorelationOldMiddleware(object):
    """
    If request_id header is present bind it to logger
    else create an ew request_id and bind it to logger
    It uses
    """

    def __init__(self):
        if not structlog.is_configured():
            configure_structlog()

    def process_request(self, request):
        setup_request_id(request)